<?php

function cli__usage() {
	cli__fatal('
Usage:
  ./paas-admin [option]

Options:
  --alive                             Do nothing
  --finished %workdir                 Mark a running process as finished
  --help                              Display this message
  --user-randpw %username             Reset user password to a random value
  --user-reg %username %password      Register a new user
  --user-setpw %username %password    Reset user password to a given value
  --validate-zip %file                Validate an appdata.zip file
');
}

function cli__fatal($str) {
	fputs(STDERR, $str."\n");
	die(1);
}

function cli__main($argv) {
	
	if (! count($argv)) {
		cli__usage();
	}
	
	for ($i = 0, $e = count($argv); $i !== $e; ++$i) {
		
		switch($argv[$i]) {
			
			case '--alive': {
				// no-op
			} break;
			
			case '--help': {
				cli__usage();
			} break;
			
			case '--finished': {
				DB::running_stop( $argv[++$i] );
			} break;
			
			case '--user-randpw': {
				$username = $argv[++$i];
				$password = substr(sha1(uniqid()), 0, 6);
				DB::user_setpass($username, $password);
				echo "${password}\n";
			} break;
			
			case '--user-reg': {
				$username = $argv[++$i];
				$password = $argv[++$i];
				if (DB::user_register($username, $password)) {
					echo "OK\n";
				} else {
					cli__fatal('Failed to register user');
				}
			} break;
			
			case '--user-setpw': {
				$username = $argv[++$i];
				$password = $argv[++$i];
				DB::user_setpass($username, $password);
			} break;
			
			case '--validate-zip': {
				$ret = paas__validate($argv[++$i]);
				if ($ret === true) {
					echo "OK\n";
				} else {
					cli__fatal('Bad validation: '.$ret);
				}
				
			} break;
			
			default: {
				cli__fatal('Unrecognised argument "'.$argv[$i].'" (try --help)');
			} break;
		}
		
	}
	
	return true;
}

function cli__processRequest() {
	die(cli__main(array_slice($_SERVER['argv'], 1)) ? 0 : 1);
}
