// app.js

$(function() {
	
	var AUTHTOKEN = null;
	var LAST_PROJECT_ID = null;
	
	//
	
	var round_to = function(value, decimals) {
		var itn = Math.pow(10, decimals);
		return (Math.round(value * itn) / itn);
	};
	
	var format_size = function(bytes) {
		if (bytes < 1024) {
			return bytes+" B";
		} else if (bytes < 1024*1024) {
			return round_to(bytes/1024, 2)+" KiB";
		} else {
			return round_to(bytes/(1024*1024), 2)+" MiB";
		}
	};
	
	var format_time = function(unix) {
		var fx = function(sz) {
			return (sz < 10) ? "0" + sz : "" + sz;
		}
		var d = new Date(unix * 1000);
		
		return (
			d.getFullYear() + "-" + fx(d.getMonth()+1) + "-" + fx(d.getDate()) +
			" " +
			fx(d.getHours()) + ":" + fx(d.getMinutes()) + ":" + fx(d.getSeconds())
		);
	};
	
	//
	
	var set_view = function(name) {
		// should be called *only* by viewstate_** functions
		$(".page").hide();
		$("#page-"+name).show();
	};
	
	var set_title = function(str) {
		$("#page-title").text("PaaS > " + str);
		$("title").text(str + " | PaaS");
	};
	
	var request_cb_wrapper = function(cb) {
		return function(ret) {
			if ((ret instanceof Object) && ('error' in ret)) {
				if (ret.error === "Session expired") {
					loginstate_setLoggedOut();
				}
				viewstate_error(ret.error);
			} else {
				cb(ret);
			}
		};		
	};
	
	var request = function(action, params, cb) {
		params['action'] = action;
		if (AUTHTOKEN !== null) {
			params['token'] = AUTHTOKEN;
		}
		$.post('backend.php', params, request_cb_wrapper(cb));
	};
	
	var request_full = function(action, params) {
		params['action'] = action;
		if (AUTHTOKEN !== null) {
			params['token'] = AUTHTOKEN;
		}
		
		var $f = $("<form method='POST' action='backend.php' target='_blank'>");
		for (var i in params) {
			$f.append( $("<input>").attr('type', 'hidden').attr('name', i).val(params[i]) );
		}
		$("body").append( $f );
		$f.submit();
	};
	
	var request_upload = function(action, formdata, cb) {
		formdata.append( 'action', action );
		if (AUTHTOKEN !== null) {
			formdata.append( 'token', AUTHTOKEN );
		}
		$.ajax({
			url: 'backend.php',
			type: 'POST',
			data: formdata,
			success: request_cb_wrapper(cb),
			
			// Raw
			cache: false,
			contentType: false,
			processData: false
		});
	};
	
	// 
	
	var viewstate_login = function() {
		$("#page-login input[type=password]").val('');
		$("#page-login input[type=text]").val('');
		set_view('login');
		set_title('Login');
	};
	
	var viewstate_waiting = function() {
		set_view('waiting');
	};
	
	var viewstate_error = function(error_str) {
		$("#page-error-message").text(error_str);
		set_view('error');
	};
	
	var viewstate_projects = function() {
		set_title('My Projects');
		set_view('waiting');
		
		request('projectlist', {}, function(ret) {
			
			$("#page-projects-new input[type=text]").val('');
			
			$("#page-projects-tbody tr").remove();
			
			$("#page-projects-invitekey").val( ret.invite_key );
			
			for (var i in ret.projects) { (function(i) {
				
				var $tr = $("<tr>").append([
					$("<td>").append(
						$("<button>").text(ret.projects[i]).click(function() {
							viewstate_viewproject( i );
						})
					),
					$("<td>").append(
						$("<button>").text("Delete").click(function() {
							viewstate_deleteproject( i );
						})
					)
				]);
				$("#page-projects-tbody").append( $tr );
				
			})(i); }
			
			set_view('projects');
		});
	};
	
	var viewstate_deleteproject = function(project_id) {
		if (confirm('Are you sure you want to delete this project?')) {
			set_view('waiting');
			request(
				'projectdelete',
				{ 'project_id': project_id },
				function(ret) {
					viewstate_projects();
				}
			);
		}
	};
	
	var viewstate_registerproject = function(title) {
		set_view('waiting');
		request(
			'projectnew',
			{ 'title': title },
			function(ret) {
				viewstate_projects();
			}
		);
	};
	
	var viewstate_startproject = function(appdata_id, project_id) {
		set_view('waiting');
		request(
			'exec',
			{ appdata_id: appdata_id },
			function(ret) {
				setTimeout(function() {
					viewstate_viewproject(project_id);
				}, 1000);
			}
		);
	}
	
	var viewstate_viewkill = function(running_id) {
		set_view('waiting');
		request(
			'kill',
			{ running_id: running_id },
			function(ret) {
				viewstate_viewproject( LAST_PROJECT_ID );
			}
		);
	};
	
	var viewstate_viewproject = function(project_id) {
		
		var display_endtime = function(row) {
			if (row['end_time'] > 0) {
				return format_time(row['end_time']);
			} else if (row['has_started'] == 1) {
				return '(running)';
			} else {
				return '(starting up)';
			}
		};
		
		var display_tasks = function(row) {
			var ret = [];
			if (row['end_time'] == '0') {
				ret.push( $("<button>")
					.text('Stop')
					.click(function() {
						viewstate_viewkill( row['id'] );
					})
				);
			}
			ret.push( $("<button>")
				.text('Logs')
				.click(function() {
					viewstate_viewlogs( row['id'] )
				})
			);
			
			return ret;
		};
		
		set_view('waiting');
		request(
			'projectinfo',
			{ 'project_id': project_id },
			function(ret) {
				set_title( ret.title );
				
				LAST_PROJECT_ID = project_id;
				
				var vermap = [];
				
				$("#page-project-tbody tr").remove();
				for (var i in ret.versions) { (function(i) {
					
					vermap[ ret.versions[i]['id'] ] = (ret.versions.length - i);
					
					var $tr = $("<tr>").append([
						$("<td>").text(
							(ret.versions.length - i)
						),
						$("<td>").append(
							$("<button>")
								.text( format_size(ret.versions[i]['filesize']) )
								.click(function() {
									viewstate_download( ret.versions[i]['id'] );
								})
						),
						$("<td>").text(
							format_time(ret.versions[i]['upload_time'])
						),
						$("<td>").append(
							$("<button>")
								.text('Start')
								.click(function() {
									viewstate_startproject(
										ret.versions[i]['id'],
										project_id
									);
								})
						)
					]);
					$("#page-project-tbody").append( $tr );
					
				})(i); } 
				
				$("#page-project-instances tr").remove();
				for (var i in ret.instances) {
					var $tr = $("<tr>").append([
						$("<td>").text(
							vermap[ ret.instances[i]['appdata_id'] ]
						),
						$("<td>").text(
							format_time( ret.instances[i]['start_time'] )
						),
						$("<td>").text( display_endtime(ret.instances[i]) ),
						$("<td>").append( display_tasks(ret.instances[i]) )
					]);
					$("#page-project-instances").append( $tr );
				}
				
				$("#page-project-apikey").val( ret.api_key );
				
				$("#page-project-upload input[type=file]")[0].value = ''; // clear files
				
				set_view('project');
			}
		);
	};
	
	var viewstate_viewlogs = function(running_id) {
		set_view('waiting');
		request(
			'logs',
			{ 'running_id': running_id },
			function(ret) {
				$("#page-logs-area").text(ret);
				
				set_view('logs');
			}
		);
	};
	
	var viewstate_download = function(db_id) {
		request_full(
			'download', 
			{ 'version_id': db_id }
		);
	};
	
	var viewstate_upload = function(formdata) {
		set_view('waiting');
		formdata.append( 'project_id', LAST_PROJECT_ID );
		
		request_upload(
			'upload',
			formdata,
			function(ret) {
				viewstate_viewproject( LAST_PROJECT_ID );
			}
		);
	};
	
	var viewstate_regenkey = function(project_id) {
		set_view('waiting');
		request(
			'regenerate',
			{ project_id: project_id },
			function(ret) {
				viewstate_viewproject( project_id );
			}
		);
	};
	
	var viewstate_logout = function() {
		set_view('waiting');
		if (AUTHTOKEN !== null) {
			request(
				'logout',
				{},
				function(ret) {
					if (ret === "ok") {
						loginstate_setLoggedOut();
						viewstate_login();
					} else {
						viewstate_error('Failed to logout.');
					}
				}
			);
		} else {
			viewstate_error('Already logged out');
		}
	}
	
	var viewstate_loggingin = function(username, password) {
		set_view('waiting');
		request(
			'login',
			{ username: username, password: password },
			function(ret) {			
				loginstate_setLoggedIn(ret.authtoken);
				viewstate_projects();
			}
		);
	};
	
	var viewstate_register = function(invite_key, username, password) {
		set_view('waiting');
		request(
			'register',
			{ invite_key: invite_key, username: username, password: password },
			function(ret) {
				loginstate_setLoggedIn(ret.authtoken);
				viewstate_projects();
			}
		);
	};
	
	var viewstate_recoverfromerror = function() {
		if (AUTHTOKEN === null) {
			viewstate_login();
		} else {
			viewstate_projects();
		}
	};
	
	//
	
	var loginstate_setLoggedIn = function(token) {
		AUTHTOKEN = token;
		$("#logout-button").show();
	};
	
	var loginstate_setLoggedOut = function() {
		AUTHTOKEN = null;
		$("#logout-button").hide();
	};
	
	//
	
	$("#page-login-form").submit(function() {
		viewstate_loggingin(
			$("#page-login-form input[type=text]").val(),
			$("#page-login-form input[type=password]").val()
		);
		return false;
	});
	
	$("#page-login-register").submit(function() {
		viewstate_register(
			$("#page-login-register-invite_key").val(),
			$("#page-login-register-username").val(),
			$("#page-login-register-password").val()
		);
		return false;
	});
	
	$("#page-projects-new").submit(function() {
		viewstate_registerproject(
			$("#page-projects-new input[type=text]").val()
		);
		return false;
	});
	
	$("#page-project-upload").submit(function() {
		viewstate_upload( new FormData( $(this)[0] ) );
		return false;
	});
	
	$("#page-project-upload-btn").click(function() {
		$("#page-project-upload").submit();
	});
	
	$("#page-project-apiform").submit(function() {
		viewstate_regenkey(LAST_PROJECT_ID);
		return false;
	});
	
	$("#page-error-recover").click(function() {
		viewstate_recoverfromerror();
	});
	
	$("#logout-button").click(function() {
		viewstate_logout();
	});
	
	$("#page-project-back").click(function() {
		viewstate_projects();
	});
	
	$("#page-logs-back").click(function() {
		viewstate_viewproject(LAST_PROJECT_ID);
	});
	
	//
	
	loginstate_setLoggedOut();
	viewstate_login();
	
	// showtime
	$(".container").css({
		'position': 'relative',
		'top': '-300px',
		'display': 'block'
	}).animate(
		{ 'top': 0 }, 
		function() {
			$("#page-login-form input[type=text]").focus()
		}
	);
});
